import json
from urllib.parse import urljoin

from requests import Session
from requests.adapters import HTTPAdapter
from requests.auth import HTTPBasicAuth
from urllib3.util.retry import Retry

from event_api_client.config import TopicHttpClientConfig as CONFIG


class TopicHttpClient(Session):
    def __init__(self):
        super().__init__()
        self.host = CONFIG['host']
        self.auth = HTTPBasicAuth(CONFIG['user'], CONFIG['password'])
        retry = Retry(
		    total=CONFIG['retry'],
            read=CONFIG['retry'],
            connect=CONFIG['retry'],
		    backoff_factor=1,
            status_forcelist=CONFIG['status_forcelist'])
        adapter = HTTPAdapter(max_retries=retry)
        self.mount('http://', adapter)
        self.mount('https://', adapter)

    def create_message_for_topic(self, topic, data, user_agent=None):
        path = '/'.join(s.strip('/') for s in ['topic', topic])
        url = urljoin(self.host, path)
        headers = {'user-agent': user_agent} if user_agent else {}
        if not isinstance(data, list):
            data = [data]
        return self.post(
            url,
            data='\n'.join(map(json.dumps, data)),
            headers=headers
        )
