import sqlite3
import os
import arrow
from abc import ABC
import numpy as np


def pacific_now():
    return arrow.utcnow().to('US/Pacific')


class SeenIPState(ABC):
    def __init__(self, name='seen_ips', path=None, rotate=1):
        self.name = name
        self.rotate_days = rotate
        self.state_path = path or os.getenv(
            'STATE_PATH', '/tmp/state')
        self.state_filename = os.path.join(self.state_path, self.name + '.db')
        if not os.path.exists(self.state_path):
            os.makedirs(self.state_path, mode=0o700)
        self.db = sqlite3.connect(self.state_filename)

    def initdb(self):
        if not os.path.exists(self.state_filename):
            self.db = sqlite3.connect(self.state_filename)

        if not self.created_stamp():
            self.db.execute("""CREATE TABLE IF NOT EXISTS state_created
                   (stamp)""")
            self.db.execute("""CREATE TABLE IF NOT EXISTS domain_ips
                   (domain, ip, PRIMARY KEY (domain, ip))""")
            self.db.execute("INSERT INTO state_created values ('%s')" % pacific_now(
            ).replace(hour=00, minute=00, second=00, microsecond=00))
            self.db.commit()

    def dump(self):
        tables = ['state_created', 'domain_ips']
        for table in tables:
            print("===== dumping contents of table %s ====== " % table)
            for r in self.db.execute("select * from %s" % table):
                print(r)

    def created_stamp(self):
        try:
            stamp_cur = self.db.execute("SELECT stamp from state_created")
            cs = stamp_cur.fetchone()
            if cs:
                return cs[0]
        except Exception as e:
            return None

    def rotate(self):
        if not self.rotate_days:
            return True
        should_rotate = 0
        try:
            if self.created_stamp():
                created_stamp = arrow.get(self.created_stamp())
                now_stamp = pacific_now()
                if (now_stamp - created_stamp).days >= self.rotate_days:
                    should_rotate = 1
            else:
                should_rotate = 1
        except Exception as e:
            should_rotate = 1

        if should_rotate:
            os.remove(self.state_filename)
            self.initdb()

    def add_ips(self, domain, ips=[], chunk=500):
        # make sure db is good
        self.initdb()
        for ip_chunk in np.array_split(ips, chunk):
            if not len(ip_chunk):
                break
            for ip in ip_chunk:
                try:
                    self.db.execute(
                        "INSERT INTO domain_ips values ('%s', '%s')" % (domain, ip))
                except Exception as e:
                    pass
            self.db.commit()

    def count_ips(self, domain):
        try:
            cur = self.db.execute(
                "select count(*) from domain_ips where domain='%s'" % (domain))
            count = cur.fetchone()
            if len(count):
                return count[0]
        except Exception as e:
            return 0
